/*****************************************************************************
IsoHex3_3.cpp
Ernest S. Pazera
08APR2000
Start a WIN32 Application Workspace, add in this file
No other libs are required
*****************************************************************************/

//////////////////////////////////////////////////////////////////////////////
//INCLUDES
//////////////////////////////////////////////////////////////////////////////
#define WIN32_LEAN_AND_MEAN  

#include <windows.h>   

//////////////////////////////////////////////////////////////////////////////
//DEFINES
//////////////////////////////////////////////////////////////////////////////
//name for our window class
#define WINDOWCLASS "ISOHEX3"
//title of the application
#define WINDOWTITLE "IsoHex 3-3"

//////////////////////////////////////////////////////////////////////////////
//PROTOTYPES
//////////////////////////////////////////////////////////////////////////////
bool Prog_Init();//game data initalizer
void Prog_Loop();//main game loop
void Prog_Done();//game clean up

//////////////////////////////////////////////////////////////////////////////
//GLOBALS
//////////////////////////////////////////////////////////////////////////////
HINSTANCE hInstMain=NULL;//main application handle
HWND hWndMain=NULL;//handle to our main window
//pens, old and new
HPEN hpenNew=NULL;
HPEN hpenOld=NULL;
//region
HRGN hrgnClip=NULL;

//////////////////////////////////////////////////////////////////////////////
//WINDOWPROC
//////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK TheWindowProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	//which message did we get?
	switch(uMsg)
	{
	case WM_DESTROY://the window is being destroyed
		{

			//tell the application we are quitting
			PostQuitMessage(0);

			//handled message, so return 0
			return(0);

		}break;
	case WM_PAINT://the window needs repainting
		{
			//a variable needed for painting information
			PAINTSTRUCT ps;
			
			//start painting
			HDC hdc=BeginPaint(hwnd,&ps);

			/////////////////////////////
			//painting code would go here
			/////////////////////////////

			//end painting
			EndPaint(hwnd,&ps);
						
			//handled message, so return 0
			return(0);
		}break;
	}

	//pass along any other message to default message handler
	return(DefWindowProc(hwnd,uMsg,wParam,lParam));
}


//////////////////////////////////////////////////////////////////////////////
//WINMAIN
//////////////////////////////////////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nShowCmd)
{
	//assign instance to global variable
	hInstMain=hInstance;

	//create window class
	WNDCLASSEX wcx;

	//set the size of the structure
	wcx.cbSize=sizeof(WNDCLASSEX);

	//class style
	wcx.style=CS_OWNDC | CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;

	//window procedure
	wcx.lpfnWndProc=TheWindowProc;

	//class extra
	wcx.cbClsExtra=0;

	//window extra
	wcx.cbWndExtra=0;

	//application handle
	wcx.hInstance=hInstMain;

	//icon
	wcx.hIcon=LoadIcon(NULL,IDI_APPLICATION);

	//cursor
	wcx.hCursor=LoadCursor(NULL,IDC_ARROW);

	//background color
	wcx.hbrBackground=(HBRUSH)GetStockObject(BLACK_BRUSH);

	//menu
	wcx.lpszMenuName=NULL;

	//class name
	wcx.lpszClassName=WINDOWCLASS;

	//small icon
	wcx.hIconSm=NULL;

	//register the window class, return 0 if not successful
	if(!RegisterClassEx(&wcx)) return(0);

	//create main window
	hWndMain=CreateWindowEx(0,WINDOWCLASS,WINDOWTITLE, WS_BORDER | WS_SYSMENU | WS_VISIBLE,0,0,320,240,NULL,NULL,hInstMain,NULL);

	//error check
	if(!hWndMain) return(0);

	//if program initialization failed, then return with 0
	if(!Prog_Init()) return(0);

	//message structure
	MSG msg;

	//message pump
	for(;;)	
	{
		//look for a message
		if(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			//there is a message

			//check that we arent quitting
			if(msg.message==WM_QUIT) break;
			
			//translate message
			TranslateMessage(&msg);

			//dispatch message
			DispatchMessage(&msg);
		}

		//run main game loop
		Prog_Loop();
	}
	
	//clean up program data
	Prog_Done();

	//return the wparam from the WM_QUIT message
	return(msg.wParam);
}

//////////////////////////////////////////////////////////////////////////////
//INITIALIZATION
//////////////////////////////////////////////////////////////////////////////
bool Prog_Init()
{
	//create a solid red pen
	hpenNew=CreatePen(PS_SOLID,0,RGB(255,0,0));

	//retrieve the client rectangle for the window
	RECT rcClient;
	GetClientRect(hWndMain,&rcClient);

	//create an elliptical region
	//hrgnClip=CreateEllipticRgn(0,0,rcClient.right,rcClient.bottom);

	POINT ptVertice[4];
	ptVertice[0].x=rcClient.right/2;
	ptVertice[0].y=0;
	ptVertice[1].x=rcClient.right;
	ptVertice[1].y=rcClient.bottom/2;
	ptVertice[2].x=rcClient.right/2;
	ptVertice[2].y=rcClient.bottom;
	ptVertice[3].x=0;
	ptVertice[3].y=rcClient.bottom/2;

	hrgnClip=CreatePolygonRgn(ptVertice,4,ALTERNATE);
	
	//borrow the dc from the main window
	HDC hdc=GetDC(hWndMain);

	//select the new pen into the dc, and keep the old one
	hpenOld=(HPEN)SelectObject(hdc,hpenNew);

	//select the clipping region into the dc
	SelectObject(hdc,hrgnClip);

	//make vertical stripes
	int nStripeX=rcClient.right/10;
	int nCount;

	//loop through and draw the stripes
	for(nCount=0;nCount<10;nCount++)
	{
		//moveto the top of the client area
		MoveToEx(hdc,nStripeX*nCount,0,NULL);

		//line to the bottom of the client area
		LineTo(hdc,nStripeX*nCount,rcClient.bottom);
	}

	//make the horizontal stripes
	int nStripeY=rcClient.bottom/10;

	//loop through and draw the stripes
	for(nCount=0;nCount<10;nCount++)
	{
		//move to the left of the client area
		MoveToEx(hdc,0,nStripeY*nCount,NULL);

		//line to the right of the client area
		LineTo(hdc,rcClient.right,nStripeY*nCount);
	}

	//return the borrowed dc to the system
	ReleaseDC(hWndMain,hdc);

	return(true);//return success
}

//////////////////////////////////////////////////////////////////////////////
//CLEANUP
//////////////////////////////////////////////////////////////////////////////
void Prog_Done()
{
	//borrow the dc from the main window
	HDC hdc=GetDC(hWndMain);

	//restore the old pen
	SelectObject(hdc,hpenOld);

	//return the dc to the system
	ReleaseDC(hWndMain,hdc);

	//delete our gdi objects
	DeleteObject(hrgnClip);
	DeleteObject(hpenNew);
}

//////////////////////////////////////////////////////////////////////////////
//MAIN GAME LOOP
//////////////////////////////////////////////////////////////////////////////
void Prog_Loop()
{
	///////////////////////////
	//main game logic goes here
	///////////////////////////
}

